@@ -9,7 +9,7 @@ import os
 from warnings import warn
 
 from reV.handlers.exclusions import ExclusionLayers
-from reV.supply_curve.points import SupplyCurveExtent
+from reV.supply_curve.extent import SupplyCurveExtent
 from reV.supply_curve.tech_mapping import TechMapping
 from reVX.wind_dirs.mean_wind_dirs_point import MeanWindDirectionsPoint
 from reVX.utilities.exclusions_converter import ExclusionsConverter
@@ -336,7 +336,7 @@ class TurbineFlicker:
             excl_fpath, building_layer, point.name,
             resolution=resolution, building_threshold=building_threshold)
 
-        with WindX(res_fpath) as f:
+        with WindX(res_fpath, log_vers=False) as f:
             dset = 'winddirection_{}m'.format(hub_height)
             wind_dir = f[dset, :, int(point['res_gid'])]
 
